var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React from "react";
import Head from "next/head";
import DefaultApp from "next/app";
import { startDsfrReact } from "./start";
import { isBrowser } from "./tools/isBrowser";
import { objectKeys } from "tsafe/objectKeys";
import marianneLightWoff2Url from "../dsfr/fonts/Marianne-Light.woff2";
import marianneItalicWoff2Url from "../dsfr/fonts/Marianne-Light_Italic.woff2";
import marianneRegularWoff2Url from "../dsfr/fonts/Marianne-Regular.woff2";
import marianneRegularItalicWoff2Url from "../dsfr/fonts/Marianne-Regular_Italic.woff2";
import marianneMediumWoff2Url from "../dsfr/fonts/Marianne-Medium.woff2";
import marianneMediumItalicWoff2Url from "../dsfr/fonts/Marianne-Medium_Italic.woff2";
import marianneBoldWoff2Url from "../dsfr/fonts/Marianne-Bold.woff2";
import marianneBoldItalicWoff2Url from "../dsfr/fonts/Marianne-Bold_Italic.woff2";
import spectralRegularWoff2Url from "../dsfr/fonts/Spectral-Regular.woff2";
import spectralExtraBoldWoff2Url from "../dsfr/fonts/Spectral-ExtraBold.woff2";
import appleTouchIcon from "../dsfr/favicon/apple-touch-icon.png";
import faviconSvg from "../dsfr/favicon/favicon.svg";
import faviconIco from "../dsfr/favicon/favicon.ico";
import faviconWebmanifestUrl from "../dsfr/favicon/manifest.webmanifest";
import { data_fr_scheme, data_fr_theme, $colorScheme } from "./colorScheme";
import { createStatefulObservable } from "./tools/StatefulObservable";
import { symToStr } from "tsafe/symToStr";
const fontUrlByFileBasename = {
    "Marianne-Light": marianneLightWoff2Url,
    "Marianne-Light_Italic": marianneItalicWoff2Url,
    "Marianne-Regular": marianneRegularWoff2Url,
    "Marianne-Regular_Italic": marianneRegularItalicWoff2Url,
    "Marianne-Medium": marianneMediumWoff2Url,
    "Marianne-Medium_Italic": marianneMediumItalicWoff2Url,
    "Marianne-Bold": marianneBoldWoff2Url,
    "Marianne-Bold_Italic": marianneBoldItalicWoff2Url,
    "Spectral-Regular": spectralRegularWoff2Url,
    "Spectral-ExtraBold": spectralExtraBoldWoff2Url
};
const $overwriteGetInitialProps = createStatefulObservable(() => undefined);
let defaultColorScheme;
export function getColorSchemeSsrUtils() {
    var _a;
    (_a = $overwriteGetInitialProps.current) === null || _a === void 0 ? void 0 : _a.call($overwriteGetInitialProps);
    $overwriteGetInitialProps.subscribe(overwriteGetInitialProps => overwriteGetInitialProps === null || overwriteGetInitialProps === void 0 ? void 0 : overwriteGetInitialProps());
    let colorScheme = undefined;
    let isNextTickCleared = false;
    function readColorSchemeFromCookie(ctx) {
        var _a, _b;
        const cookie = (_a = ctx.req) === null || _a === void 0 ? void 0 : _a.headers.cookie;
        colorScheme =
            (_b = (cookie === undefined ? undefined : readColorSchemeInCookie(cookie))) !== null && _b !== void 0 ? _b : (() => {
                switch (defaultColorScheme) {
                    case "light":
                    case "dark":
                        return defaultColorScheme;
                    case "system":
                        return undefined;
                }
            })();
        isNextTickCleared = false;
        process.nextTick(() => {
            if (!isNextTickCleared) {
                console.error([
                    `WARNING: ${symToStr({
                        getColorSchemeHtmlAttributes
                    })} should be called just after`,
                    `${symToStr({ getColorSchemeSsrUtils })}, in the same event loop tick!`
                ].join(" "));
            }
        });
    }
    function getColorSchemeHtmlAttributes() {
        isNextTickCleared = true;
        if (colorScheme === undefined) {
            return {};
        }
        $colorScheme.current = colorScheme;
        return {
            [data_fr_scheme]: colorScheme,
            [data_fr_theme]: colorScheme
        };
    }
    return { readColorSchemeFromCookie, getColorSchemeHtmlAttributes };
}
export function withAppDsfr(App, params) {
    const { preloadFonts = [] } = params, startDsfrReactParams = __rest(params, ["preloadFonts"]);
    if (isBrowser) {
        startDsfrReact(startDsfrReactParams);
    }
    else {
        defaultColorScheme = startDsfrReactParams.defaultColorScheme;
    }
    function AppWithDsfr(props) {
        return (React.createElement(React.Fragment, null,
            React.createElement(Head, null,
                process.env.NODE_ENV !== "development" &&
                    objectKeys(fontUrlByFileBasename)
                        .filter(fileBasename => preloadFonts.includes(fileBasename))
                        .map(fileBasename => fontUrlByFileBasename[fileBasename])
                        .map(url => (React.createElement("link", { key: url, rel: "preload", href: url, as: "font", crossOrigin: "anonymous" }))),
                React.createElement("link", { rel: "apple-touch-icon", href: appleTouchIcon.src }),
                React.createElement("link", { rel: "icon", href: faviconSvg.src, type: "image/svg+xml" }),
                React.createElement("link", { rel: "shortcut icon", href: faviconIco.src, type: "image/x-icon" }),
                React.createElement("link", { rel: "manifest", href: faviconWebmanifestUrl, crossOrigin: "use-credentials" }),
                React.createElement("style", null, `:root { color-scheme: ${$colorScheme.current}; }`)),
            React.createElement(App, Object.assign({}, props))));
    }
    Object.keys(App).forEach(staticMethod => (AppWithDsfr[staticMethod] = App[staticMethod]));
    $overwriteGetInitialProps.current = () => {
        AppWithDsfr.getInitialProps = async (appContext) => {
            var _a, _b;
            const initialProps = await ((_a = App.getInitialProps) !== null && _a !== void 0 ? _a : DefaultApp.getInitialProps)(appContext);
            if (!isBrowser) {
                $colorScheme.current =
                    (_b = (() => {
                        var _a;
                        const cookie = (_a = appContext.ctx.req) === null || _a === void 0 ? void 0 : _a.headers.cookie;
                        return cookie === undefined ? undefined : readColorSchemeInCookie(cookie);
                    })()) !== null && _b !== void 0 ? _b : (() => {
                        switch (startDsfrReactParams.defaultColorScheme) {
                            case "dark":
                            case "light":
                                return startDsfrReactParams.defaultColorScheme;
                            case "system":
                                return "light";
                        }
                    })();
            }
            return Object.assign({}, initialProps);
        };
    };
    AppWithDsfr.displayName = AppWithDsfr.name;
    return AppWithDsfr;
}
function readColorSchemeInCookie(cookie) {
    const parsedCookies = Object.fromEntries(cookie
        .split(/; */)
        .map(line => line.split("="))
        .map(([key, value]) => [key, decodeURIComponent(value)]));
    if (!(data_fr_theme in parsedCookies)) {
        return undefined;
    }
    const colorScheme = parsedCookies[data_fr_theme];
    return (() => {
        switch (colorScheme) {
            case "light":
            case "dark":
                return colorScheme;
            default:
                return undefined;
        }
    })();
}
//# sourceMappingURL=nextJs.js.map